/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * ================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://forceten.tidalwave.it
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: FinderSupport.java,v c9bf78d7704e 2010/02/14 03:07:46 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.location.elmo.impl;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;
import it.tidalwave.util.Finder.SortCriterion;
import it.tidalwave.util.Finder.SortDirection;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.geo.Coordinate;
import it.tidalwave.geo.Range;
import it.tidalwave.geo.location.GeoLocation;
import it.tidalwave.geo.location.GeoLocation.Finder;
import it.tidalwave.semantic.Wrapper;

/***********************************************************************************************************************
 *
 * FIXME: possibly get rid of this, and instead use GeoFinder. I think that only qualifiedName needs to be moved
 * and to integrate Wrapper. But Wrapper could be moved to the GeoFinderProvider.
 * 
 * @author Fabrizio Giudici
 * @version $Id: FinderSupport.java,v c9bf78d7704e 2010/02/14 03:07:46 fabrizio $
 *
 **********************************************************************************************************************/
public abstract class FinderSupport implements Finder // TODO: use FinderSupport from util
  {
    @Nonnull
    private final String description;

    @Nonnegative
    private int firstResult = 0;

    @Nonnegative
    private int maxResults = 9999999; // gets inolved in some math, so can't use Integer.MAX_VALUE

    @CheckForNull
    protected String displayName;

    @CheckForNull
    protected String path;

    @CheckForNull
    protected Range range;
    
    @Nonnull
    private final List<GeoLocation> result = new ArrayList<GeoLocation>();

    private boolean computed = false;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public FinderSupport (final @Nonnull String description)
      {
        this.description = description;
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnegative
    public int count()
      {
        compute();
        return result.size();
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Finder from (final @Nonnegative int firstResult)
      {
        this.firstResult = firstResult;
        return this;
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Finder max (final @Nonnegative int maxResults)
      {
        this.maxResults = maxResults;
        return this;
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Finder withDisplayName (final @Nonnull String displayName)
      {
        this.displayName = displayName;
        return this;
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Finder withQualifiedName (final @Nonnull String path)
      {
        this.path = path;
        return this;
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Finder closeTo (final @Nonnull Coordinate coordinate, final @Nonnegative double radius)
      {
        range = Range.on(coordinate).withRadius(radius);
        return this;
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public <X> it.tidalwave.util.Finder<X> ofType (final @Nonnull Class<X> type)
      {
        throw new UnsupportedOperationException("Not supported.");
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public GeoLocation result()
      throws NotFoundException
      {
        compute();
        final String message = String.format("%s - name: %s - path: %s", description, displayName, path);

        switch (result.size())
          {
            case 0:
              throw new NotFoundException(message);

            case 1:
              return result.get(0);

            default:
              throw new RuntimeException("More than one result: " + message);
          }
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    public GeoLocation firstResult()
      throws NotFoundException
      {
        max(1);
        compute();

        if (result.isEmpty())
          {
            final String message = String.format("%s - name: %s - path: %s", description, displayName, path);
            throw new NotFoundException(message);
          }

        return result.get(0);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public List<? extends GeoLocation> results()
      {
        compute();
        return result.subList(firstResult, Math.min(result.size(), firstResult + maxResults));
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    public Finder sort (final @Nonnull SortCriterion criterion,
                        final @Nonnull SortDirection direction)
      {
        throw new UnsupportedOperationException("Not supported yet."); // TODO
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    public Finder sort (final @Nonnull SortCriterion criterion)
      {
        throw new UnsupportedOperationException("Not supported yet."); // TODO
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected abstract List<GeoLocation> doCompute();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private final synchronized void compute()
      {
        if (!computed)
          {
            result.clear();
            result.addAll(Wrapper.wrap(doCompute()));
            computed = true;
          }
      }
  }
