/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * ================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://forceten.tidalwave.it
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: RangeFinder.java,v c9bf78d7704e 2010/02/14 03:07:46 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.location.elmo.impl.finder;

import javax.annotation.Nonnull;
import java.util.List;
import java.util.ArrayList;
import org.openide.util.Lookup;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.util.Finder;
import it.tidalwave.util.FinderSupport;
import it.tidalwave.geo.Range;
import it.tidalwave.geo.finder.spi.GeoFinderProvider;
import it.tidalwave.geo.geocoding.GeoCoderEntity;
import it.tidalwave.geo.location.GeoLocation;
import it.tidalwave.geo.location.GeoSchema;
import it.tidalwave.geo.location.elmo.impl.ElmoGeoCoderEntityProxyFactory;
import it.tidalwave.geo.location.elmo.impl.ElmoGeoLocationFactory;
import it.tidalwave.geo.location.elmo.impl.GeoCoderEntityProxy;
import static it.tidalwave.geo.GeoContainer.GeoContainer;
import static it.tidalwave.geo.geocoding.GeoCoderEntity.GeoCoderEntity;

/***********************************************************************************************************************
 *
 * @author Fabrizio Giudici
 * @version $Id: RangeFinder.java,v c9bf78d7704e 2010/02/14 03:07:46 fabrizio $
 *
 **********************************************************************************************************************/
@ServiceProvider(service=GeoFinderProvider.class)
public class RangeFinder extends GeoFinderProvider<GeoLocation, Range>
  {
    public RangeFinder()
      {
        super(Range.class, GeoLocation.class);
      }

    @Override @Nonnull
    protected Finder<GeoLocation> find (final @Nonnull Range range,
                                        final @Nonnull Class<? extends GeoLocation> type,
                                        final @Nonnull Lookup parameters)
      {
        return new FinderSupport<GeoLocation>("")
          {
            @Override @Nonnull
            protected List<GeoLocation> doCompute()
              {
                final GeoSchema schema = FinderUtil.findGeoSchema(parameters);
                final List<GeoLocation> result = new ArrayList<GeoLocation>();

                for (final GeoCoderEntity gcEntity : range.as(GeoContainer).find(GeoCoderEntity).results())
                  {
                    final GeoCoderEntityProxy gcEntityProxy =
                        ElmoGeoCoderEntityProxyFactory.findOrCreate(schema, gcEntity);
                    result.add(ElmoGeoLocationFactory.findOrCreate(schema, gcEntityProxy));
                  }

                return result;
              }
          };
      }
  }
